/*
	language.c
	Routines for language-handling.
	14.3.99 tn
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <glib.h>
#include <gtk/gtk.h>
#include "xcdroast.h"
#include "main.h"

/* global language setting */
gint language;

language_list_t **langarray;
gint langcount;

gchar **mesgarray;
gint mesgcount;
gint mesgcount2;

gchar **helparray;
gint helpcount;
gint helpcount2;


/* returns the text-string for the current selected language */

gchar *text(gint string_id) {
gchar *tmppnt;

	tmppnt = mesgarray[langcount*string_id+language];
	if (tmppnt == NULL) {
		/* if no mesg found, try default language 0 */
		tmppnt = mesgarray[langcount*string_id+0];
	}
	if (tmppnt == NULL) {
		return "- N.A. -";
	} else {
		return tmppnt;
	}
}


/* returns the text-string for the current selected language */

gchar *help(gint string_id) {
gchar *tmppnt;

	tmppnt = helparray[langcount*string_id+language];
	if (tmppnt == NULL) {
		/* if no mesg found, try default language 0 */
		tmppnt = helparray[langcount*string_id+0];
	}
	if (tmppnt == NULL) {
		return "";
	} else {
		return tmppnt;
	}
}


/* takes the lines from the lang.defs-file and sorts them into a
   memory structure */

void parse_line(gchar *line) {
gchar tmp[MAXLINE];
gchar tmp2[MAXLINE];
gchar *tmppnt;
gchar *tmppnt2;
gint n;

	strip_string(line);

	/* skip comments and empty lines */
	if (line[0] == '#') return;
	if (line[0] == '\0') return;
	
	if (strncmp(line,"LANG",4) == 0) {
		strcpy(tmp,line);
		tmppnt = strtok(tmp,":");
		if (tmppnt == NULL) return;
		tmppnt = strtok(NULL,";");
		if (tmppnt == NULL) return;
		strcpy(tmp,tmppnt);
		strip_string(tmp);
		
		/* now we have found a language string */
		langarray[langcount]=g_new(language_list_t,1);
		langarray[langcount]->langstring = g_strdup(tmp);
		langarray[langcount]->codes = NULL;

		/* now look for the iso-language-codes */
		tmppnt = strtok(NULL, "");
		if (tmppnt != NULL) {
			strcpy(tmp,tmppnt);
			strip_string(tmp);
			tmppnt = strtok(tmp,",");
			while (tmppnt != NULL) {
				strcpy(tmp2, tmppnt);
				strip_string(tmp2);
				langarray[langcount]->codes = g_list_append(langarray[langcount]->codes, g_strdup(tmp2));
				tmppnt = strtok(NULL,",");
			}	
		}

		langcount++;

		if (langcount >= MAXLANG) {
			g_warning("maximal number of language definitions hit. Aborting...\n");
			gtk_exit(1);
		}	

		return;
	}

	/* all strings now are message-definitions */
	
	strcpy(tmp,line);
	tmppnt = strtok(tmp,"_:");
	if (tmppnt == NULL) return;
	tmppnt = strtok(NULL,"_:");
	if (tmppnt == NULL) return;
	strcpy(tmp,tmppnt);

	/* if default-language "0" is found, switch to new block */
	if (strcmp(tmp,"0") == 0) {
		mesgcount++;
		mesgcount2=0;
	}
	tmppnt = strtok(NULL,"");
	if (tmppnt == NULL) return;
	strcpy(tmp,tmppnt);
	strip_string(tmp);

	/* now strip quotes from string */	
	if (*tmp == '\"') {
		tmppnt2 = tmp+1;
	} else {
		tmppnt2 = tmp;
	} 
	if (tmp[strlen(tmp)-1] == '\"') {
		tmp[strlen(tmp)-1] = '\0';
	}
	strcpy(tmp2,tmppnt2);
	escape_parse(tmp2);

	n = strlen(tmp2)+1;
	mesgarray[mesgcount*langcount+mesgcount2]=g_new(gchar,n);
	strcpy(mesgarray[mesgcount*langcount+mesgcount2],tmp2);
	
	mesgcount2++;

	if (mesgcount2 >= MAXENTRIES) {
		g_warning("maximal number of text definitions hit. Aborting...\n");
		gtk_exit(1);
	}	
}


/* takes the lines from the lang.defs-file and sorts them into a
   memory structure */

void parse_helpline(gchar *line) {
gchar tmp[MAXLINE];
gchar tmp2[MAXLINE];
gchar *tmppnt;
gchar *tmppnt2;
gint n;

	strip_string(line);

	/* skip comments and empty lines */
	if (line[0] == '#') return;
	if (line[0] == '\0') return;
	
	/* all strings are help-definitions */
	
	strcpy(tmp,line);
	tmppnt = strtok(tmp,"_:");
	if (tmppnt == NULL) return;
	tmppnt = strtok(NULL,"_:");
	if (tmppnt == NULL) return;
	strcpy(tmp,tmppnt);

	/* if default-language "0" is found, switch to new block */
	if (strcmp(tmp,"0") == 0) {
		helpcount++;
		helpcount2=0;
	}
	tmppnt = strtok(NULL,"");
	if (tmppnt == NULL) return;
	strcpy(tmp,tmppnt);
	strip_string(tmp);

	/* now strip quotes from string */	
	if (*tmp == '\"') {
		tmppnt2 = tmp+1;
	} else {
		tmppnt2 = tmp;
	} 
	if (tmp[strlen(tmp)-1] == '\"') {
		tmp[strlen(tmp)-1] = '\0';
	}
	strcpy(tmp2,tmppnt2);
	escape_parse(tmp2);

	n = strlen(tmp2)+1;
	helparray[helpcount*langcount+helpcount2]=g_new(gchar,n);
	strcpy(helparray[helpcount*langcount+helpcount2],tmp2);
	
	helpcount2++;

	if (helpcount2 >= MAXENTRIES) {
		g_warning("maximal number of help texts hit. Aborting...\n");
		gtk_exit(1);
	}	
}


/* load language-definitions into memory */

void load_langfile(gchar *filename) {
FILE *fd;
gchar line[MAXLINE];

	/* allocate memory */
	mesgarray = g_new0(gchar *,MAXENTRIES+1);
	mesgcount = -1;
	mesgcount2 = 0;
	langarray = g_new0(language_list_t *,MAXLANG+1);
	langcount = 0;

	dodebug(1,"Opening language file %s\n", filename);
	if ((fd = fopen(filename,"r")) == NULL) {
		g_warning("Error opening %s\n", filename);
		g_warning("Without language file X-CD-Roast cannot be started - aborting\n");
		gtk_exit(1);
	}

	for (;;) {
		if (fgets(line,MAXLINE,fd) == NULL)
			break;
		dodebug(11,"language: %s",line);
		parse_line(line);
	}		
	
	fclose(fd);
}


/* load language-definitions into memory */

void load_helpfile(gchar *filename) {
FILE *fd;
gchar line[MAXLINE];

	/* allocate memory */
	helparray = g_new0(gchar *,MAXENTRIES+1);
	helpcount = -1;
	helpcount2 = 0;

	dodebug(1,"Opening language help file %s\n", filename);
	if ((fd = fopen(filename,"r")) == NULL) {
		g_warning("Error opening %s\n", filename);
		g_warning("Without language help file X-CD-Roast cannot be started - aborting\n");
		gtk_exit(1);
	}

	for (;;) {
		if (fgets(line,MAXLINE,fd) == NULL)
			break;
		dodebug(11,"language.help: %s",line);
		parse_helpline(line);
	}		
	
	fclose(fd);
}


/* scan the LC_ALL/LANG-environment variable and find matching language */
/* otherwise set to english */

void scan_lang() {
gchar *lang;
GList *iso;
gint i;

	/* check LC_ALL first */
	lang = getenv("LC_ALL");
	if (lang == NULL) {
		/* fall back to alternative var */
		lang = getenv("LANG");
	}
	if (lang != NULL) {
		/* now scan for id string */
		i = 0;
		while(langarray[i] != NULL) {
			iso = g_list_first(langarray[i]->codes);
			while (iso) {
				if (strncasecmp(lang, (gchar *)iso->data, 
				    strlen((gchar *)iso->data)) == 0) {
					/* found match */
					language = i;
					dodebug(1, "Enabling language setting: %s (can be overidden by configfile)\n", lang);
				}
				iso = iso->next;
			}
			i++;
		}
	}

	/* no match found? */
	if (language == -1) {
		/* use english as default */
		language = 0;
	}
}

